# The AIConsole Project
#
# Copyright 2023 10Clouds
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
    
class NoOpenAPIKeyException(Exception):
    def __str__(self):
        return "No API key provided or is invalid. " \
               "You must set the environment variable OPENAI_API_KEY=<API-KEY>. " \
               "Generate API keys in the OpenAI web interface. " \
               "See https://platform.openai.com/account/api-keys for details."
